function Listener(listener: MethodListener<any>) {
    return function (target: any, propertyKey: string, descriptor: PropertyDescriptor) {
        let originalMethod = descriptor.value;
        //wrapping the original method
        descriptor.value = function (...args: any[]) {
            console.log("wrapped function: before invoking " + propertyKey);
            let result = originalMethod.apply(this, args);
            listener.onMethodInvoked(result);
            console.log("wrapped function: after invoking " + propertyKey);
            return result;
        }
    }
}

interface MethodListener<T> {
    onMethodInvoked(result: T): void;
}

class MyListener implements MethodListener<any> {
    onMethodInvoked(result: any): void {
        console.log("MyListener#onMethodInvoked: " + result);
    }
}

class TaskRunner {
    @Listener(new MyListener())
    runTask(taskName: string): any {
        console.log("runTask invoked: " + taskName);
        return "the task result"
    }
}

console.log("-- creating an instance --");
let taskRunner = new TaskRunner();
console.log("-- invoking TaskRunner --");
let output = taskRunner.runTask("task input");
console.log(output);